<?php

class ActionSquadro
{
    private $plateau;

    public function __construct(PlateauSquadro $plateau)
    {
        $this->plateau = $plateau;
    }

    /**
     * Vérifie si une pièce peut être jouée (c'est-à-dire si elle n'est pas vide).
     */
    public function estJouablePiece($x, $y): bool
    {
        $piece = $this->plateau->getPiece($x, $y);
        return $piece->getCouleur() !== PieceSquadro::VIDE;
    }

    /**
     * Joue une pièce en la déplaçant, en gérant les collisions, les retournements et les sorties.
     */
    public function jouePiece($x, $y): void
    {
        if (!$this->estJouablePiece($x, $y)) {
            throw new Exception("Cette pièce ne peut pas être jouée.");
        }

        $piece = $this->plateau->getPiece($x, $y);
        list($destX, $destY) = $this->plateau->getCoordDestination($x, $y);

        // Vérifie si le mouvement est en ligne droite
        if ($x != $destX && $y != $destY) {
            throw new Exception("Le mouvement doit être en ligne droite.");
        }

        // Gestion des collisions
        $this->gererReculAdverses($x, $y, $destX, $destY);

        // Mise à jour des coordonnées ou inversement de direction
        if ($this->estPointRetournement($destX, $destY, $piece)) {
            $piece->inverseDirection();
        } elseif ($this->estSortieValide($destX, $destY)) {
            $this->sortPiece($x, $y);
        } else {
            $this->plateau->setPiece(PieceSquadro::initVide(), $x, $y);
            $this->plateau->setPiece($piece, $destX, $destY);
        }
    }

    /**
     * Gère le recul des pièces adverses lors d'un mouvement.
     */
    private function gererReculAdverses($startX, $startY, $destX, $destY): void
    {
        $dx = $destX - $startX;
        $dy = $destY - $startY;
        $steps = max(abs($dx), abs($dy));

        for ($i = 1; $i <= $steps; $i++) {
            $x = $startX + ($dx / $steps) * $i;
            $y = $startY + ($dy / $steps) * $i;

            $adverse = $this->plateau->getPiece($x, $y);
            if ($adverse->getCouleur() !== PieceSquadro::VIDE &&
                $adverse->getCouleur() !== $this->plateau->getPiece($startX, $startY)->getCouleur()) {
                $this->reculerPiece($x, $y);
            }
        }
    }

    /**
     * Recule une pièce adverse au début de son parcours.
     */
    public function reculerPiece($x, $y): void
    {
        $piece = $this->plateau->getPiece($x, $y);
        $piece->inverseDirection();
        list($destX, $destY) = $this->plateau->getCoordDestination($x, $y);
        $this->plateau->setPiece(PieceSquadro::initVide(), $x, $y);
        $this->plateau->setPiece($piece, $destX, $destY);
    }

    /**
     * Retire une pièce du plateau lorsqu'elle a terminé son parcours.
     */
    public function sortPiece($x, $y): void
    {
        $this->plateau->setPiece(PieceSquadro::initVide(), $x, $y);
    }

    /**
     * Vérifie si une pièce atteint un point de retournement.
     */
    private function estPointRetournement($x, $y, $piece): bool
    {
        if ($piece->getCouleur() === PieceSquadro::BLANC && $x === 6) {
            return true;
        }
        if ($piece->getCouleur() === PieceSquadro::NOIR && $y === 0) {
            return true;
        }
        return false;
    }

    /**
     * Vérifie si une pièce sort du plateau.
     */
    private function estSortieValide($x, $y): bool
    {
        return $x < 0 || $x >= 7 || $y < 0 || $y >= 7;
    }

    /**
     * Vérifie si une couleur a remporté la partie.
     */
    public function remporteVictoire($couleur): bool
    {
        $piecesSorties = 0;
        for ($i = 0; $i < 7; $i++) {
            for ($j = 0; $j < 7; $j++) {
                $piece = $this->plateau->getPiece($i, $j);
                if ($piece->getCouleur() === $couleur && $this->estSortieValide($i, $j)) {
                    $piecesSorties++;
                }
            }
        }
        return $piecesSorties >= 5; // Une couleur gagne si 5 pièces sont sorties
    }
}
?>